/*
  File name: cda.h
  Created by: Ljubomir Buturovic
  Created: 02/12/2003
  Purpose: declarations for principal components analysis and Fisher's
  linear discriminant.
*/

/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "pcp.h"

/*
  Collect user input and call Principal Component Analysis linear
  feature extractor pca() (type == PDR_PCA) or calculate Fisher's
  linear discriminant(s) (type == PDR_FISHER), using the training data
  set.
*/
void p_cda(int type, int *errc, char **xname);

/*
  Calculate principal components transformation matrix on 'dset'.
  
  Return -1 in case of error and set 'errc'. Errors are malloc()
  errors and LERR_ITMAX in eigen().
*/
float **pca(struct dataset *dset, int *errc);

/*
  Calculate Fisher's linear discriminant (transformation matrix) on
  'dset'.
  
  Return -1 in case of error and set 'errc'. Errors are malloc()
  errors and LERR_ITMAX in eigen().
*/
float **fld(struct dataset *dset, int *errc);
