/*
  File name: crit.h
  Created by: Ljubomir Buturovic
  Created: 07/13/2004
  Purpose: declarations of criterion functions and gradient for
  various optimization programs, such as simplex() and conjg().
*/

/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#ifndef CRIT_H
#define CRIT_H

/*
  Prototype of criterion function for optimization programs such as
  simplex().
*/
typedef float (crit_func)(float *x, int n, int iteration, void *parameters, int *errc);

/*
  Prototype of criterion function for optimization programs which use
  'double *' argument, such as conjg().
*/
typedef double (crit_dfunc)(double *x, int n, int iteration, void *parameters, int *errc);

/*
  Prototype of gradient for optimization programs which use gradient,
  such as conjg().
*/
typedef void (crit_gradient)(double *x, int n, double *grad, void *parameters, int *errc);

#endif
