/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#ifndef EMAP_H
#define EMAP_H

#include <stdio.h>
#include "dataset.h"

/*
  Default max. number of iterations in simplex() for EMAP.
*/
#define EMAP_ITMAX      1000

typedef float (*emap_type)(float [], struct dataset *, int, int, int, unsigned int,
			   int *, FILE *, FILE *);

/*
  Compute EMAP transformation matrix for dataset 'dset' into
  'dr'-dimensional space. 'kmin' and 'kmax' are parameters used for
  the Bayes error estimation.
  
  The function uses PCA (Principal Component Analysis) as initial
  point for the simplex optimization if number of input vectors is
  greater than dimensionality; otherwise it uses SVD (Singular Value
  Decomposition). The max. number of iterations is fixed to
  EMAP_MAX_ITER.

  In case of success, return the transformation matrix. In case of
  failure, return NULL and set 'errc'. The errors are EINVAL for bad
  arguments, memory allocation errors,
*/
float **emap(struct dataset *dset, int dr, int kmin, int kmax, int *errc);

#endif
