/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#ifndef HASH_UTIL_H
#define HASH_UTIL_H

#include "hash.h"

#define MAX_INT_LENGTH     31   /* maximum length of an integer, +1 */

hash_t *hashCreate(void);

/*
  Return number of elements in 'hashtable'.
*/
int hashCount(hash_t *hashtable);

/*
  Put integer 'value' under 'key' into 'hashtable'. Return the stored
  value. NOTE: this function will return INT_MAX if the operation
  fails. Therefore it should not be used for hashtables where INT_MAX
  is a legal value.
*/
int hashPutInt(hash_t *hashtable, int key, int value);

char *hashPutString(hash_t *hashtable, char *key, char *value);

hash_t *hashPutHashtable(hash_t *hashtable, char *key, hash_t *value);

/*
  Get integer value corresponding to 'key' from 'hashtable'. NOTE:
  this function will return INT_MAX if 'key' is not in the
  hashtable. That means that this function should not be used for
  hashtables where INT_MAX is a legal value.
*/
int hashGetInt(hash_t *hashtable, int key);

char *hashGetString(hash_t *hashtable, char *key);

hash_t *hashGetHashtable(hash_t *hashtable, char *key);

/*
  Put integer 'value' under string 'key' into 'hashtable'. Return the
  stored value. NOTE: this function will return INT_MAX if the
  operation fails. Therefore it should not be used for hashtables
  where INT_MAX is a legal value.
*/
int hashPutStrInt(hash_t *hashtable, char *key, int value);

/*
  Get integer value corresponding to string 'key' from
  'hashtable'. NOTE: this function will return INT_MAX if 'key' is not
  in the hashtable. That means that this function should not be used
  for hashtables where INT_MAX is a legal value.
*/
int hashGetStrInt(hash_t *hashtable, char *key);

/*
  Remove 'key' from 'hashtable'.
*/
void *hashRemove(hash_t *hashtable, void *key);

/*
  Free all 'hashtable' elements and then free 'hashtable' itself.

  NOTE: hash.c source code says hash_free() is obsolete. Don't
  understand why.
*/
void hashDelete(hash_t *hashtable);

#endif
