/*
  File name: lerr.h
  Created by: Ljubomir Buturovic
  Created: 04/18/2002
  Purpose: error code definitions (in addition to <errno.h> codes).
*/

/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#ifndef LERR_H
#define LERR_H

/*
  Unrecognized file format.
*/
#define LERR_FILE_FORMAT             -100

/*
  Conjugate gradient optimization procedure unable to obtain descent
  direction.
*/
#define LERR_DESCENT                 -101

/*
  Line search failure in optimization programs.
*/
#define LERR_LNSEARCH                -102

/*
  Internal software error.
*/
#define LERR_INTERNAL                -103

/*
  This error code indicates that a function has attempted to apply a
  classifier (model) to a data set inconsistent with the model. For
  example, in case of a parametric quadratic classifier, the number of
  coefficients is different from the number of linear and quadratic
  terms.
*/
#define LERR_INCONSISTENT_MODEL      -104

/*
  Similar to LERR_INCONSISTENT_MODEL, for linear classifiers.
*/
#define LERR_INCONSISTENT_LIN        -105

/*
  Singular covariance matrix detected.
*/
#define LERR_SINGCOV                 -106

/*
  Attempted inversion of a singular matrix.
*/
#define LERR_SINGULAR                -107

/*
  Too many iterations in an iterative procedure.
*/
#define LERR_ITMAX                   -108

/*
  Variable number of columns in input file.
*/
#define LERR_VAR_NCOL                -109


#endif 
