/*
  File name: p_lind.c
  Created by: Ljubomir Buturovic
  Created: 08/04/2004
  Purpose: menu functions for linear discriminant learning.
*/

/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "lin.h"
#include "lind.h"
#include "widrow.h"
#include "lau.h"
#include "pau.h"
#include "bagging.h"

/*
  Read linear discriminant parameters from stdin.
*/
static int lin_input(FILE *outdev, int c, int nvec, char **fname)
{
  int status = 0;

  clear_screen();
  cursor_on();
  *fname = input_filename(LD_MSG, PCP_LIN, outdev);
  return status;
}

/*
  Collect input parameters from the user and call lind_learn() which
  performs the training.
  
  In case of failure, set 'errc'. If error is file access error, set
  'xname' to the name of the file which triggered the error.
*/
void p_lind_learn(int *errc, char **xname, int *dbg)
{
  int    status;
  char   *fname;
  float  **model;
  FILE   *outdev;
  FILE   *fdbg = (FILE *) 0;

  if (tds)
    {
      outdev = stdout;
      fflush(outdev);
      if (*dbg > 0)
	fdbg = fopen(PCP_DBG, "a");
      status = lin_input(outdev, tds->c, tds->nv, &fname);
      if (status == 0)
	{
	  model = lind_learn(tds, errc, fdbg);
	  /*
	    TBD: Widrow-Hoff learning disabled for now, due to problems
	    with setting the parameters. ljb, 08/11/2004.
	  */
	  /*model = widrow_learn(tds, errc, fdbg);*/
	  if (!model)
	    status = -1;
	  else
	    status = fmx_save(model, tds->c, tds->d+1, fname, 0);
	  if (status)
	    {
	      *errc = errno;
	      *xname = strdup(fname);
	    }
	  else
	    {
	      /*
		Apply the model to TDS and display results.
	      */
	      *errc = dataset_lin_predict(tds, model);
	      if (!*errc)
		{
		  predict_disp(tds, 0, PALG_LIN);
		  pwait();
		}
	    }
	}
      else
	{
	  *errc = errno;
	  *xname = strdup(fname);
	}
    }
  else
    *errc = PERR_UNDEFINED_TDS;
}

