/*
  Module name: pcl.c
  Created by: Ljubomir Buturovic
  Created: 10/03/2003
  Purpose: main() program for testing PCL (Pattern Classification
  Library).
*/

/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "dataset.h"
#include "pcl_svm.h"

#define SVM_FILE  "iris.svm"

int main()
{
  int   i;
  int   j;
  int   d;
  int   c;
  int   errc;
  int   nmodels;
  int   nexp;
  int   retval;
  unsigned int seed;
  int   nd[3];
  int   **fn;
  char  *fnames[3];
  char  *fname;
  char  *pcheck;
  float *weights;
  struct dataset *dset;
  struct pcl_svm_parameters *parameters;
  struct svm_model **models;

  d = 4;
  c = 3;
  nmodels = 10;
  nexp = 5;
  retval = 0;
  nd[0] = 50;
  nd[1] = 50;
  nd[2] = 50;
  fnames[0] = strdup("iris1.dat");
  fnames[1] = strdup("iris2.dat");
  fnames[2] = strdup("iris3.dat");
  dset = load_dataset(d, c, nd, fnames, DATASET_FF_RAW, &errc, &fname);
  retval = -1;
  if (dset)
    {
      fname = strdup(SVM_FILE);
      parameters = calloc(1, sizeof(struct pcl_svm_parameters));

      /*
	Set Support Vector Machine parameters.
       */
      parameters->kernel_type = POLY;
      parameters->C = 1.0;
      parameters->degree = 3.0;
      parameters->gamma = 0.25;
      
      pcheck = pcl_svm_check(dset, parameters);
      if (pcheck)
	printf("Bad parameters: %s\n", pcheck);
      else
	{
	  /*
	    Learn.
	  */
	  models = pcl_svm_learn(dset, nmodels, parameters, &errc);
	  if (models)
	    /* 
	       Save SVM.
	    */
	    pcl_svm_save(fname, models, nmodels);
	  else
	    printf("pcl_svm_learn() returns NULL; errc: %d\n", errc);
	  /*
	    Load SVM.
	   */
	  models = pcl_svm_load(SVM_FILE, &nmodels, &weights);
	  if (models)
	    {
	      printf("The file contains %d models.\n", nmodels);
	      /*
		Predict.
	       */
	      pcl_svm_predict(dset, models, nmodels, weights, &errc);
	      if (!errc)
		{
		  for (i = 0; i < dset->nv; i++)
		    printf("prediction[%d]: %d\n", i+1, dset->prediction[i]+1);
		  /*
		    Cross-validation.
		  */
		  seed = 1;
		  fn = pcl_svm_xlearn(dset, 0, 0, 0, nexp, seed, nmodels, parameters, &errc);
		  if (fn)
		    {
		      for (i = 0; i < nexp; i++)
			{
			  for (j = 0; j < c; j++)
			    printf("fn[%d][%d]: %12d ", i, j, fn[i][j]);
			  printf("\n");
			  for (j = 0; j < c; j++)
			    printf("en[%d][%d]: %12.5f ", i, j, (float) fn[i][j]/dset->nd[j]);
			  printf("\n");
			}
		      retval = 0;
		    }
		  else
		    printf("pcl_svm_xlearn() returns NULL.\n");
		}
	      else
		printf("pcl_svm_predict() returns error %d\n", errc);
	    }
	  else	
	    printf("pcl_svm_load() returns NULL; errno: %d\n", errno);
	}
    }
  else
    {
      printf("Error loading dataset; errc: %d", errc);
      if (fname)
	printf("; fname: %s.\n", fname);
      else
	printf(".\n");
      retval = -1;
    }
  return retval;
}
