/*
  File name: pkbd.c
  Created by: Ljubomir Buturovic
  Created: 06/09/1999
  Purpose: keyboard reader.
*/

/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

static char rcsid[] = "$Id: pkbd.c,v 1.15 2005/06/02 02:24:17 ljubomir Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <ctype.h>
#include "pcp.h"

/*
  Read one character from keyboard, no echo. Return it in 'key'.
*/
int read_keyboard(int *key)
{
  int  status;
  char input;
  char *line;
  struct termios buf;
  struct termios save_termios;
  struct stat stat_buf;

  status = fstat(STDIN_FILENO, &stat_buf);
  if (!status)
    {
      /*
	If it is real file, read 'key' from there.
      */
      if (S_ISREG(stat_buf.st_mode))
	{
	  line = malloc(P_MAX_LINE_LEN);
	  if (fgets(line, P_MAX_LINE_LEN-1, stdin))
	    {
	      tolower(line[0]);
	      if (line[0] == 'x')
		*key = 12;
	      else
		*key = line[0]-'a'+1;
	    }
	  else
	    *key = 12;
	  free(line);
	}
      else
	{
	  /*
	    Save original state.
	  */
	  status = tcgetattr(STDIN_FILENO, &save_termios);
	  if (status >= 0)
	    {
	      buf = save_termios;
	      buf.c_lflag &= ~(ECHO | ICANON);
	      buf.c_cc[VMIN] = 1;
	      buf.c_cc[VTIME] = 0;
	      status = tcsetattr(STDIN_FILENO, TCSAFLUSH, &buf);
	      if (status >= 0)
		{
		  read(STDIN_FILENO, &input, 1);
		  input = tolower(input);
		  /*
		    Accept Escape and carriage return as exit codes.
		  */
		  if (input == 27 || input == '\n')
		    input = 12;
		  else
		    input = input-'a'+1;
		  if (input >= 12) 
		    input = 12;
		  *key = input;
		  /*
		    Restore original state.
		  */
		  tcsetattr(STDIN_FILENO, TCSAFLUSH, &save_termios);
		}
	    }
	}
    }
  return status;
}

/*
  Wait for any character, no echo.
*/
void pwait(void)
{
  int  key;
  char *cTmp;
  struct stat stat_buf;

  cTmp = rcsid;
  fstat(STDIN_FILENO, &stat_buf);
/*
  If stdin is real file, return.
*/
  if (!S_ISREG(stat_buf.st_mode))
    read_keyboard(&key);
}

