/*
  File name: widrow.h
  Created by: Ljubomir Buturovic
  Created: 08/10/2004
  Purpose: declaration of linear discriminant learning function using
  Widrow-Hoff algorithm.
*/

/*
  Copyright 2004 Ljubomir J. Buturovic

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

/*
  Calculate linear discriminant classifier using Widrow-Hoff (LMS)
  algorithm.  The returned value is c by d+1 matrix (number of classes
  by number of features plus 1). The last column contains biases.

  The computations follow Algorithm 10 in Section 5.8.4, in
  R. O. Duda, P. E. Hart and D. G. Stork, Pattern Classification,
  Second Edition, John Wiley & Sons, Inc., 2001.

  In case of error, return NULL and set 'errc'. The errors are EINVAL
  if 'dset' is NULL, and memory allocation errors.
*/
float **widrow_learn(struct dataset *dset, int *errc, FILE *fdbg);
